% Copyright (c) <2014>, <Boston University>
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
% 1. Redistributions of source code must retain the above copyright
%    notice, this list of conditions and the following disclaimer.
% 2. Redistributions in binary form must reproduce the above copyright
%    notice, this list of conditions and the following disclaimer in the
%    documentation and/or other materials provided with the distribution.
% 3. All advertising materials mentioning features or use of this software
%    must display the following acknowledgement:
%    This product includes software developed by the <organization>.
% 4. Neither the name of the <organization> nor the
%    names of its contributors may be used to endorse or promote products
%    derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY <COPYRIGHT HOLDER> ''AS IS'' AND ANY
% EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%--------------------------------------------------------------------------
% function runSegmentationPerCell(imageRoot, initialSegmentRoot, datasetSubdirs, ...
%     datasetPreprocessing, segmentationMethod, ...
%     imageOutRoot, segmentRoot, bComplement)
%--------------------------------------------------------------------------
% History Log
%--------------------------------------------------------------------------
%
% 8/12/2014 - Boston University's Image and Video Computing group released
% this code
%
%--------------------------------------------------------------------------

function runSegmentationPerCell(imageRoot, datasetSubdirs, datasetPreprocessing, segmentationMethod, segmentRoot)

    validateInputs();
    initOutput();

    %% Run segmentation
    % for each data set, for each file
    % load the image, perform preprocessing on image
    % run segmentation algorithm
    % output segmentation

    for d=1:size(datasetSubdirs,1)
        imgPath = [imageRoot datasetSubdirs{d}];     
        for f=1:size(imageFiles{d},1) 
        %parfor f=1:size(imageFiles{d},1) % For parallel processing
            imageFilename = imageFiles{d}{f};
            imgFile = [imgPath imageFilename];
            image = readImage(imgFile, preprocessingMethod);
            segmentation = segmentationMethod(image, image);
            segmentation = fillLargestBlob(segmentation, true);
            segmentFilename = ['seg_' imageFilename];
            segFile = [segmentPath segmentFilename];
            imwrite(segmentation, segFile);
        end
    end

    function totalFiles = validateInputs()
        %% Basic input validation
        if ~exist('imageRoot', 'var') || ~exist('datasetSubdirs', 'var')
            error('Input directories or data set names undefined');
        end

        if ~exist('segmentationMethod', 'var') || ~isa(segmentationMethod, 'function_handle')
            error('Segmentation method given is not a function handle');
        end

        if ~exist('datasetPreprocessing', 'var') 
            error('Data preprocessing parameters not given');
        else
            preprocessingMethod = [];
            if isempty(datasetPreprocessing)
                % do nothing
            elseif isa(datasetPreprocessing, 'function_handle')
                preprocessingMethod = datasetPreprocessing;
            elseif isa(datasetPreprocessing, 'cell')
                if isempty(datasetPreprocessing)
                    preprocessingMethod = [];
                else
                    preprocessingMethod = datasetPreprocessing{1};
                end
            else
                error('Data preprocessing is not a cell array or a function handle');
            end
        end

        if ~exist('preprocessingMethod', 'var') || (~isempty(preprocessingMethod) && ~isa(preprocessingMethod, 'function_handle'))
            error('Preprocessing method given is not a function handle');
        end

        % convert whatever format of input for the paths into cell arrays
        % replace back slashes with forward slashes and add trailing slashes
        imageRoot = cleanPath(imageRoot);
        datasetSubdirs = cleanPath(makeCellArray(datasetSubdirs));

        % make sure all the data sets exist under the gold standard
        % and get all the filenames
        [imageFiles totalFiles] = checkFoldersAndFiles(imageRoot, datasetSubdirs, []);
        segmentFiles = imageFiles;
        for d=1:length(segmentFiles)
            for f=1:length(segmentFiles{d})
                segmentFiles{d}{f}=['seg_' segmentFiles{d}{f}];
            end
        end
    end

    function initOutput()
        %% Check existence of output folders and files. Create directories if needed
        if exist(segmentRoot, 'dir')
            %warning('Folder for output segmentations exists');
        else
            mkdir(segmentRoot)
        end

        segmentRoot = cleanPath(segmentRoot);

        for d=1:length(datasetSubdirs) 
            segmentPath = [segmentRoot datasetSubdirs{d}];
            if exist(segmentPath, 'dir')
                warning('Output segmentation path %s already exists', segmentPath);
                rmdir(segmentPath, 's');
                mkdir(segmentPath);
            else
                mkdir(segmentPath);
            end
        end
    end
end
